#!/bin/sh

usage()
{
    echo "
USAGE:
  $(basename $0) [OPTIONS] [OUTPUT]

  OPTIONS:
    -b|--board <ax2|ax1|qemu>  -- select which output to build
    -s|--sign  <key> <cert>                -- produce output.sig signature
    --release                              -- do not append date to version
    -h|--help

  OUTPUT:
    filename of output. (e.g. alpine.tar.gz)
    if signing, also output.sig (e.g. alpine.tar.gz.sig)
"
}

error()
{
    echo "$@" >&2
    exit 1
}

############
# Veriables
############
alpine_version=$(cat alpine-version)
alpine_version_atmark=$(cat alpine-version)
atmark_version=$(cat atmark-version)
signkey=
signcert=
timestamp_version=1

# parse args
while [ $# -ne 0 ]
do
    case "$1" in
	"-b"|"--board")
	    [ $# -lt 2 ] && error "$1 requires an argument"
	    case "$2" in
		"ax2")
		    board=ax2
		    arch=aarch64
		    suffix=x2
		    platform="linux/aarch64"
		    dtb=armadillo_iotg_g4.dtb
		    ;;
		"qemu")
		    board=qemu
		    suffix=qemu
		    arch=x86_64
		    platform="linux/x86_64"
		    ;;
		"ax1")
		    board=ax1
		    suffix=x1
		    arch=armhf
		    platform="linux/arm"
		    ;;
		*)
		    echo "unknown board($2)"
		    usage
		    exit 1
		    ;;
	    esac
	    shift
	    ;;
	"--alpine_version")
	    [ $# -lt 2 ] && error "$1 requires an argument"
	    alpine_version="$2"
	    shift
	    ;;
	"--release")
	    timestamp_version=
	    ;;
	"-s"|"--sign")
	    [ $# -lt 3 ] && error "$1 requires [key cert] arguments"
	    signkey="$2"
	    signcert="$3"
	    [ -r "$signkey" ] || error "key $signkey not readable"
	    [ -r "$signcert" ] || error "cert $signcert not readable"
	    shift 2
	    ;;
	"-h"|"--help")
	    usage
	    exit 0
	    ;;
	-*)
	    echo "Invalid option $1" >&2
	    usage >&2
	    exit 1
	    ;;
	*)
	    outdir="$(dirname "$(readlink -f "$1")")"
	    output="$(basename "$1")"
	    extension="tar${output##*.tar}"
	    if [ "$extension" = "tar$output" ]; then
		    echo "Output file \"$output\" must end in .tar.*" >&2
		    exit 1
	    fi
	    ;;
    esac
    shift
done

# set default
if [ -z "$board" ]; then
    echo "use default(board=ax2)"
    echo "use default(arch=aarch64)"
    board=ax2
    arch=aarch64
    suffix=x2
    platform="linux/aarch64"
    dtb=armadillo_iotg_g4.dtb
fi
if [ -z "$output" ]; then
    outdir="$(pwd)"
    output="baseos-$suffix-ATVERSION.tar.zst"
    extension=tar.zst
    echo "use default(outdir=$outdir)"
    echo "use default(output=$output)"
fi
# enter release mode if tag has been made
if [ -e .git ] && git describe --exact-match >/dev/null 2>&1 &&
        git diff-index --ignore-submodules=untracked --quiet HEAD; then
    timestamp_version=
fi

packages="$(cat $board/packages | grep -v ^# | xargs)"
alpine_container=alpine-${alpine_version#v}-${arch}

############
# Functions
############
get_scripts()
{
    [ -e .git ] && git submodule update -i
    cp -rL "$PWD/submodules/alpine-make-rootfs" "$workdir/" \
	    || error "alpine-make-rootfs not initialized?"

    local UID=$(id -u)
    local GID=$(id -g)

    cp -L $board/fixup "$workdir"
    chmod +x "$workdir/fixup"

    cat > "$workdir/repositories" <<EOF
http://download.atmark-techno.com/alpine/$alpine_version_atmark/atmark
https://dl-cdn.alpinelinux.org/alpine/$alpine_version/main
https://dl-cdn.alpinelinux.org/alpine/$alpine_version/community
EOF

    cat > "$workdir/build-alpine-internal.sh" <<EOF
#!/bin/sh -e

echo "start build-alpine-internal.sh"

echo "
start install"
./install

echo "
start fixup"
./fixup "$atmark_version${timestamp_version:+.$(date +%Y%m%d)}"

# add link to dtb
if [ -n "$dtb" ]; then
	if ! [ -e "/boot/$dtb" ]; then
		echo "/boot/$dtb does not exist !!!" >&2
		exit 1
	fi
	ln -s "$dtb" /boot/armadillo.dtb
fi

[ -e /etc/atmark-release ] && cp /etc/atmark-release .
EOF
    chmod +x "$workdir/build-alpine-internal.sh"

    cat > "$workdir/build-alpine.sh" <<EOF
#!/bin/sh -e

echo 'du -sk "\$rootfs" | cut -f 1 > '"\$(pwd)"'/rootfs_footprint_kbyte' \
    >> "alpine-make-rootfs/alpine-make-rootfs"

exit_handler() {
    local ret=\$?
    echo "build-alpine.sh done"
    exit \$ret
}

trap exit_handler INT EXIT

# build apparently uses the host repo even with --repositories-file option
cp -v repositories /etc/apk/
apk update

alpine-make-rootfs/alpine-make-rootfs \
    --branch $alpine_version \
    --timezone 'Asia/Tokyo' \
    --packages "$packages" \
    --script-chroot \
    --repositories-file repositories \
    "rootfs.${extension%.zst}" ./build-alpine-internal.sh

rm -rf rootfs
chown "$UID:$GID" "rootfs.${extension%.zst}" rootfs_footprint_kbyte
EOF
    chmod +x "$workdir/build-alpine.sh"

    cp -L $board/install "$workdir/install"
    chmod +x "$workdir/install"
}

rm_scripts()
{
    rm -rf "$workdir"
}

exit_handler()
{
    local ret=$?

    set +e

    cd "$outdir"
    rm_scripts
    if [ $ret -eq 0 ]; then
	echo "done."
    else
	echo "error occured."
    fi

    return $ret
}

int_handler()
{
    exit_handler
    local ret=$?

    echo "caught interapt"
    exit $ret
}

#######
# Main
#######
if ! docker image inspect "$alpine_container" > /dev/null; then
    [ -e .git ] && git submodule update -i
    echo "trying to build required alpine container"
    (
        cd submodules/containers &&
            BUILDARCHS="$arch" VERSION="${alpine_version#v}" ./build.sh
    ) || error "Could not build alpine container"
fi

trap exit_handler EXIT
trap int_handler INT

workdir="$(mktemp -d -t alpine-build-rootfs.XXXXXX)"

get_scripts
cp -rL $board/resources "$workdir"

cd "$workdir"
docker run --privileged --rm -v "$workdir":/build -w /build \
       --platform "$platform" \
       "$alpine_container" /build/build-alpine.sh \
	|| error "Failed building image"

if [ "${output%.zst}" != "$output" ]; then
    zstd --rm -10 "$workdir/rootfs.tar"
fi

rootfs_footprint=$(cat $workdir/rootfs_footprint_kbyte)000
tarball_size=$(stat -c %s "$workdir/rootfs.$extension")
if [ "${output%ATVERSION*}" != "$output" ]; then
	[ -e "$workdir/atmark-release" ] \
		|| error "ATVERSION required for output name, but atmark-release was not created"
	atmark_version=$(cat "$workdir/atmark-release")
	output="${output%ATVERSION*}${atmark_version}${output#*ATVERSION}"
fi

mv "$workdir/rootfs.$extension" "$workdir/$output"

if [ -n "$signkey" ]; then
    openssl cms -sign -in "$workdir/$output" -out "$workdir/$output.sig" \
	    -signer "$signcert" -inkey "$signkey" -outform DER \
	    -nosmimecap -binary \
	    || error "Could not sign tarball"
fi

echo "name,footprint[byte],tarball[byte],packages" > "$workdir/footprint.csv"
echo "${output%.$extension},$rootfs_footprint,$tarball_size,$packages" >> "$workdir/footprint.csv"

echo "============================================"
printf "footprint[byte]  tarball[byte]  packages\n"
printf "%15d  %13d  %s\n" \
       "$rootfs_footprint" \
       "$tarball_size" \
       "$packages"
echo "============================================"


[ -e "$workdir/$output.sig" ] && mv "$workdir/$output.sig" "$outdir"
mv "$workdir/$output" "$workdir/footprint.csv" "$outdir"
